﻿using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Polly;
using Polly.Caching;
using Polly.Registry;

namespace SelectiveCachingValuesInsideHttpResponsesInPolly.Controllers
{
    [Produces("application/json")]
    [Route("api/[controller]")]
    public class CatalogController : Controller
    {
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly IPolicyRegistry<string> _policyRegistry;

        public CatalogController(IHttpClientFactory httpClientFactory, IPolicyRegistry<string> policyRegistry)
        {
            _httpClientFactory = httpClientFactory;
            _policyRegistry = policyRegistry;
        }

        [HttpGet("{id}")]
        public async Task<IActionResult> Get(int id)
        {
            string requestEndpoint = $"inventory/{id}";
            var httpClient = _httpClientFactory.CreateClient("RemoteServer");

            var cachePolicy = _policyRegistry.Get<CachePolicy<(int, HttpStatusCode)>>("cacheOnly200Ok");
            Context policyExecutionContext = new Context($"GetInventoryById-{id}");

            (int ItemsInStock, HttpStatusCode StatusCode) itemsInStockAndStatusCode =
                await cachePolicy.ExecuteAsync(context => MakeRequest(), policyExecutionContext);

            async Task<(int count, HttpStatusCode statusCode)> MakeRequest()
            {
                HttpResponseMessage response = await httpClient.GetAsync(requestEndpoint);

                if (response.IsSuccessStatusCode)
                {
                    int itemsInStock = JsonConvert.DeserializeObject<int>(await response.Content.ReadAsStringAsync());
                    return (itemsInStock, response.StatusCode);
                }

                // it doesn't matter what int you use here, it won't be cached as the StatusCode is not a success code
                return (0, response.StatusCode);
            }

            if (itemsInStockAndStatusCode.StatusCode != HttpStatusCode.OK)
            {
                 return StatusCode((int)itemsInStockAndStatusCode.StatusCode);
            }
            return Ok(itemsInStockAndStatusCode.ItemsInStock);
        }

    }
}
