﻿using System;
using System.Net;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Polly;
using Polly.Caching;
using Polly.Caching.Memory;
using Polly.Registry;

namespace SelectiveCachingValuesInsideHttpResponsesInPolly
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {
            services.AddMemoryCache();
            services.AddSingleton<IAsyncCacheProvider, MemoryCacheProvider>();

            IPolicyRegistry<string> registry = services.AddPolicyRegistry();

            services.AddHttpClient("RemoteServer", client =>
            {
                client.BaseAddress = new Uri("http://localhost:5000/api/");
                client.DefaultRequestHeaders.Add("Accept", "application/json");
            });

            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_1);
        }

        public void Configure(IApplicationBuilder app, IHostingEnvironment env,
            IAsyncCacheProvider cacheProvider, IPolicyRegistry<string> registry)
        {
            Func<Context, (int value, HttpStatusCode StatusCode), Ttl> cacheOnly200OkFilter =
                (context, result) => new Ttl(
                    timeSpan: result.StatusCode == HttpStatusCode.OK ? TimeSpan.FromSeconds(15) : TimeSpan.Zero,
                    slidingExpiration: true
                );

            IAsyncPolicy<(int, HttpStatusCode)> cachePolicy =
                Policy.CacheAsync<(int, HttpStatusCode)>(
                    cacheProvider.AsyncFor<(int, HttpStatusCode)>(), 
                    new ResultTtl<(int, HttpStatusCode)>(cacheOnly200OkFilter),
                    onCacheError: null);

            registry.Add("cacheOnly200Ok", cachePolicy);

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseMvc();
        }
    }
}
